package at.bestsolution.rsd.rt.rs;

import java.lang.annotation.Annotation;
import java.lang.reflect.Type;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.ZonedDateTime;

import javax.ws.rs.ext.ParamConverter;
import javax.ws.rs.ext.ParamConverterProvider;
import javax.ws.rs.ext.Provider;

@Provider
public class JavaTimeParamConverterProvider implements ParamConverterProvider {

	@SuppressWarnings("unchecked")
	@Override
	public <T> ParamConverter<T> getConverter(Class<T> rawType, Type genericType, Annotation[] annotations) {
		if( rawType == LocalDate.class ) {
			return (ParamConverter<T>) new LocalDateParamConverter();
		} else if( rawType == LocalTime.class ) {
			return (ParamConverter<T>) new LocalTimeParamConverter();
		} else if( rawType == LocalDateTime.class ) {
			return (ParamConverter<T>) new LocalDateTimeParamConverter();
		} else if( rawType == ZonedDateTime.class ) {
			return (ParamConverter<T>) new ZonedDateTimeParamConverter();
		}
		
		return null;
	}
	
	static class ZonedDateTimeParamConverter implements ParamConverter<ZonedDateTime> {

		@Override
		public ZonedDateTime fromString(String value) {
			return value == null ? null : ZonedDateTime.parse(value);
		}

		@Override
		public String toString(ZonedDateTime value) {
			return value == null ? null : value.toString();
		}
		
	}
	
	static class LocalDateParamConverter implements ParamConverter<LocalDate> {

		@Override
		public LocalDate fromString(String value) {
			return value == null ? null : LocalDate.parse(value);
		}

		@Override
		public String toString(LocalDate value) {
			return value == null ? null : value.toString();
		}
		
	}

	static class LocalTimeParamConverter implements ParamConverter<LocalTime> {

		@Override
		public LocalTime fromString(String value) {
			return value == null ? null : LocalTime.parse(value);
		}

		@Override
		public String toString(LocalTime value) {
			return value == null ? null : value.toString();
		}
		
	}
	
	static class LocalDateTimeParamConverter implements ParamConverter<LocalDateTime> {

		@Override
		public LocalDateTime fromString(String value) {
			return value == null ? null : LocalDateTime.parse(value);
		}

		@Override
		public String toString(LocalDateTime value) {
			return value == null ? null : value.toString();
		}
		
	}
}
