package at.bestsolution.rsd.rt.rs;

import java.io.IOException;
import java.io.OutputStream;
import java.util.function.BiConsumer;

import javax.json.Json;
import javax.json.stream.JsonGenerator;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.StreamingOutput;

public class RestStreamingOutput<T> implements StreamingOutput {
	private T data;
	private BiConsumer<T, JsonGenerator> consumer;
	
	private static final boolean DEBUG = false;
	
	public RestStreamingOutput(T data, BiConsumer<T, JsonGenerator> consumer) {
		this.data = data;
		this.consumer = consumer;
	}
	
	@Override
	public void write(OutputStream output) throws IOException, WebApplicationException {
		JsonGenerator generator = Json.createGenerator(DEBUG ? new OutputStreamDelegate(output) : output);
		consumer.accept(data, generator);
		generator.flush();
		generator.close();
	}

	static class OutputStreamDelegate extends OutputStream {
		private final OutputStream out;
		
		public OutputStreamDelegate(OutputStream out) {
			this.out = out;
		}
		@Override
		public void write(int b) throws IOException {
			out.write(b);
			System.err.write(b);
		}
		
		@Override
		public void flush() throws IOException {
			out.flush();
			System.err.flush();
		}
		
		@Override
		public void write(byte[] b) throws IOException {
			out.write(b);
			System.err.write(b);
		}
		
		@Override
		public void write(byte[] b, int off, int len) throws IOException {
			out.write(b, off, len);
			System.err.write(b, off, len);
		}
		
		@Override
		public void close() throws IOException {
			out.close();
		}
	}
}
